
import os
VER = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))

XSV_DELIMITER = '\t'
THARAVU_VER = f'{VER}/tharavu'
DEFAULT_DICTIONARY_FILES = [f'{THARAVU_VER}/chorkuviyal.22mar22.tsv.gz', f'{THARAVU_VER}/chorpirappiyal-agarathi.tsv.gz']
CHELLUMAI_ADAVU_FILE = f'{THARAVU_VER}/orungurri-chellumai-cholloadai.tsv'
